using System.Collections.Immutable;
using Upsilon.BaseTypes.ScriptTypeInterfaces;
using Upsilon.Binder;
using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.BaseTypes.ScriptFunction
{
    internal class ScriptRuntimeFunction : ScriptFunction, IScopeOwner
    {
        public BoundBlockStatement                 Block           { get; }
        public ImmutableArray<BoundVariableSymbol> Parameters      { get; }
        public EvaluationScope                     EvaluationScope { get; }

        public ScriptRuntimeFunction(ImmutableArray<BoundVariableSymbol> parameters, BoundBlockStatement block,
            EvaluationScope evaluationScope)
        {
            Parameters      = parameters;
            Block           = block;
            EvaluationScope = evaluationScope;
        }

        public override ScriptType Run(Diagnostics diagnostics, ScriptType[] variables, Script script, EvaluationScope scope, TextSpan span)
        {
            var innerEvaluator = new Evaluator.Evaluator(diagnostics, EvaluationScope, script);
            for (var i = 0; i < Parameters.Length; i++)
            {
                var parameterVariable = Parameters[i];
                var parameterValue    = variables[i];
                innerEvaluator.Scope.CreateLocal(parameterVariable.VariableSymbol, parameterValue);
            }
            return innerEvaluator.EvaluateNode(Block);
        }
    }
}