using System;
using System.Collections;
using System.Collections.Generic;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.ScriptTypeInterfaces;
using Upsilon.Evaluator;
using Upsilon.Exceptions;
using Upsilon.Text;

namespace Upsilon.BaseTypes.UserData
{
    internal class ListUserData : ScriptType, IUserData, IIterable, ILengthType
    {
        public IList List { get; }

        public ListUserData(IList list)
        {
            List = list;
        }

        public ScriptType Get(Diagnostics diagnostics, TextSpan span, ScriptType index, EvaluationScope scope)
        {
            int i;
            if (index.Type == Type.String)
            {
                var str = (ScriptString)index;
                i = int.Parse(str.Value) - 1;
            }
            else if (index.Type == Type.Number)
            {
                var ind = (Number.ScriptNumberLong) index;
                i = (int) ind.Value - 1;
            }
            else
            {
                return null;
            }
            return List[i].ToScriptType();
        }

        public void Set(Diagnostics diagnostics, TextSpan span, ScriptType scriptIndex, ScriptType value)
        {
            var index = -1;
            if (scriptIndex.Type == Type.Number || scriptIndex.Type == Type.Unknown)
            {
                index = Convert.ToInt32(scriptIndex.ToCSharpObject());
            }
            else if (scriptIndex.Type == Type.String)
            {
                index = int.Parse(((ScriptString) scriptIndex).Value);
            }
            else
            {
                var (i, pos) = diagnostics.ScriptString.GetLinePosition(span.Start);
                var line = diagnostics.ScriptString.GetLine(i);
                throw new ScriptRuntimeException(
                    $"Tried indexing a CSharp list with a value that's not an integer. Value: {scriptIndex.ToCSharpObject()}",
                    i, pos, line);
            }

            index--;
            if (index > List.Count)
            {
                if (index == List.Count)
                {
                    List.Add(value.ToCSharpObject());
                }
                else
                {
                    throw new Exception(
                        $"Tried adding an item to a list at position {index}, but list is only of length {List.Count}");
                }
            }
            else
            {
                List[index] = value.ToCSharpObject();
            }
        }

        public override Type Type => Type.UserData;
        public override object ToCSharpObject()
        {
            return List;
        }

        public override System.Type GetCSharpType()
        {
            return List.GetType();
        }

        public ScriptType GetValueFromIndex(ScriptType index)
        {
            var num = (ScriptNumberLong)index;
            return List[(int) num.Value].ToScriptType();
        }

        public IEnumerator<ScriptType> GetScriptEnumerator()
        {
            for (int i = 0; i < List.Count; i++)
            {
                yield return new ScriptNumberLong(i);
            }
        }

        public ScriptNumberLong Length()
        {
            return new ScriptNumberLong(List.Count);
        }

        protected bool Equals(ListUserData other)
        {
            return List.Equals(other.List);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() == List.GetType())
            {
                return List.Equals(obj);
            }
            if (obj.GetType() != this.GetType()) return false;
            return Equals((ListUserData) obj);
        }

    }
}