using System.Collections;
using System.Collections.Generic;
using Upsilon.BaseTypes;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundBinaryExpression : BoundExpression
    {
        public BoundBinaryExpression(BoundBinaryOperator op, BoundExpression leftExpression,
            BoundExpression rightExpression, Type type, TextSpan span) : base(span)
        {
            Operator = op;
            LeftExpression = leftExpression;
            RightExpression = rightExpression;
            Type = type;
        }

        public override BoundKind Kind => BoundKind.BoundBinaryExpression;
        public override IEnumerable<BoundNode> GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= LeftExpression.Span.Start && characterPosition <= LeftExpression.Span.End)
            {
                foreach (var exp in LeftExpression.GetNodeAtPosition(characterPosition))
                {
                    yield return exp;
                }
            }
            if (characterPosition >= RightExpression.Span.Start && characterPosition <= RightExpression.Span.End)
            {
                foreach (var exp in RightExpression.GetNodeAtPosition(characterPosition))
                {
                    yield return exp;
                }
            }
            yield return this;
        }

        public override Type Type { get; }

        public BoundBinaryOperator Operator { get; }
        public BoundExpression LeftExpression { get; }
        public BoundExpression RightExpression { get; }
    }
}