using System.Collections.Generic;
using Upsilon.BaseTypes;
using Upsilon.Parser;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundIndexExpression : BoundExpression
    {
        public BoundIndexExpression(BoundExpression identifier, BoundExpression index, Type type, TextSpan span) : base(span)
        {
            Identifier = identifier;
            Index = index;
            Type = type;
        }

        public BoundExpression Identifier { get; }
        public BoundExpression Index { get; }

        public override BoundKind Kind => BoundKind.BoundIndexExpression;
        public override IEnumerable<BoundNode> GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= Identifier.Span.Start && characterPosition <= Identifier.Span.End)
                foreach (var boundNode in Identifier.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (characterPosition >= Index.Span.Start && characterPosition <= Index.Span.End)
                foreach (var boundNode in Index.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            yield return this;
        }

        public override Type Type { get; }
    }

    public class BoundFullStopIndexExpression : BoundExpression
    {
        public BoundExpression Expression { get; }
        public string Index { get; }

        public BoundFullStopIndexExpression(BoundExpression expression, string index, Type type, TextSpan span) : base(span)
        {
            Expression = expression;
            Index = index;
            Type = type;
        }

        public override BoundKind Kind => BoundKind.BoundFullstopIndexExpression;
        public override IEnumerable<BoundNode> GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= Expression.Span.Start && characterPosition <= Expression.Span.End)
                foreach (var boundNode in Expression.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            yield return this;
        }

        public override Type Type { get; }
    }
}