using System.Collections.Generic;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundIfStatement : BoundStatement
    {
        public BoundIfStatement(BoundExpressionStatement condition, BoundBlockStatement block, TextSpan span): base(span)
        {
            Condition = condition;
            Block     = block;
        }

        public BoundIfStatement(BoundExpressionStatement condition, BoundBlockStatement block, BoundElseStatement elseStatement, TextSpan span) : base(span)
        {
            Condition     = condition;
            Block         = block;
            ElseStatement = elseStatement;
        }
        public BoundIfStatement(BoundExpressionStatement condition, BoundBlockStatement block, BoundIfStatement nextElseIf, TextSpan span) : base(span)
        {
            Condition     = condition;
            Block         = block;
            NextElseIf = nextElseIf;
        }

        public override BoundKind Kind => BoundKind.BoundIfStatement;
        public override IEnumerable<BoundNode> GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= Condition.Span.Start && characterPosition <= Condition.Span.End)
                foreach (var boundNode in Condition.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (characterPosition >= Block.Span.Start && characterPosition <= Block.Span.End)
                foreach (var boundNode in Block.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (NextElseIf != null && characterPosition >= NextElseIf.Span.Start && characterPosition <= NextElseIf.Span.End)
                foreach (var boundNode in NextElseIf.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (ElseStatement != null && characterPosition >= ElseStatement.Span.Start && characterPosition <= ElseStatement.Span.End)
                foreach (var boundNode in ElseStatement.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            yield return this;
        }

        public BoundExpressionStatement Condition { get; }
        public BoundBlockStatement Block { get; }
        public BoundIfStatement NextElseIf { get; }
        public BoundElseStatement ElseStatement { get; }
    }

    public class BoundElseStatement : BoundStatement
    {
        public BoundBlockStatement Block { get; }

        public BoundElseStatement(BoundBlockStatement block, TextSpan span) : base(span)
        {
            Block = block;
        }

        public override BoundKind Kind => BoundKind.BoundElseStatement;
        public override IEnumerable<BoundNode> GetNodeAtPosition(int characterPosition)
        {
            if (Block != null && characterPosition >= Block.Span.Start && characterPosition <= Block.Span.End)
                foreach (var boundNode in Block.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            yield return this;
        }
    }
}