using System.Collections.Generic;
using Upsilon.Binder.VariableSymbols;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundNumericForStatement : BoundStatement
    {
        public VariableSymbol Variable { get; }
        public BoundExpression BoundStart { get; }
        public BoundExpression BoundStop { get; }
        public BoundExpression BoundStep { get; }
        public BoundBlockStatement Block { get; }

        public BoundNumericForStatement(VariableSymbol variable, BoundExpression boundStart,
            BoundExpression boundStop, BoundExpression boundStep, BoundBlockStatement block, TextSpan span) : base(span)
        {
            Variable = variable;
            BoundStart = boundStart;
            BoundStop = boundStop;
            BoundStep = boundStep;
            Block = block;
        }

        public override BoundKind Kind => BoundKind.BoundNumericForStatement;
        public override IEnumerable<BoundNode> GetNodeAtPosition(int characterPosition)
        {
            if (characterPosition >= BoundStart.Span.Start && characterPosition <= BoundStart.Span.End)
                foreach (var boundNode in BoundStart.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (characterPosition >= BoundStop.Span.Start && characterPosition <= BoundStop.Span.End)
                foreach (var boundNode in BoundStop.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (characterPosition >= BoundStep.Span.Start && characterPosition <= BoundStep.Span.End)
                foreach (var boundNode in BoundStep.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            if (characterPosition >= Block.Span.Start && characterPosition <= Block.Span.End)
                foreach (var boundNode in Block.GetNodeAtPosition(characterPosition))
                    yield return boundNode;

            yield return this;

        }
    }
}