using System;
using System.Collections.Immutable;
using System.Linq;
using Type = Upsilon.BaseTypes.Type;

namespace Upsilon.Binder.VariableSymbols
{
    public class InternalFunctionVariableSymbol : FunctionVariableSymbol
    {
        public class InternalFunctionParameter
        {
            public InternalFunctionParameter(string name, Type type, bool isOptional)
            {
                ValidTypes = type;
                IsOptional = isOptional;
                Name = name;
            }

            public string Name { get; }
            public Type ValidTypes { get; }
            public bool IsOptional { get; }
        }

        public InternalFunctionParameter[] FunctionParameters { get; }
        private int MinimalParametersRequired { get; }

        public InternalFunctionVariableSymbol(string name, bool local, Type resultType, InternalFunctionParameter[] functionParameters)
            : base(name, local, resultType)
        {
            FunctionParameters = functionParameters;
            MinimalParametersRequired = functionParameters.Count(x => !x.IsOptional);
        }

        public override (bool IsValid, string Error,
            BoundExpression WrongParameter) ValidateParameters(ImmutableArray<BoundExpression> callingParameters)
        {
            if (callingParameters.Length < MinimalParametersRequired
                || callingParameters.Length > FunctionParameters.Length)
            {
                return (false,
                    $"Invalid number of parameters for function '{Name}'. Expected {FunctionParameters.Length}, got {callingParameters.Length}",
                    null);
            }

            for (var i = 0; i < callingParameters.Length; i++)
            {
                var functionParameter = FunctionParameters[i];
                var callingParameter = callingParameters[i];
                if (callingParameter.Type == Type.Unknown || callingParameter.Type == Type.Nil)
                    continue;

                if (!functionParameter.ValidTypes.HasFlag(callingParameter.Type))
                {
                    return (false,
                        $"Unexpected variable passed to internal function at variable {i + 1}. " +
                        $"Expected one of the following: {functionParameter.ValidTypes.ToString()}, got: '{callingParameter.Type}'",
                        callingParameter);
                }
            }

            return (true, null, null);
        }
    }
}