using System.Collections.Generic;
using Upsilon.Text;

namespace Upsilon.Parser
{
    public class SyntaxToken : SyntaxNode
    {
        public SyntaxToken(SyntaxKind kind, int position, string text, object value)
        {
            Kind  = kind;
            Span  = new TextSpan(position, text.Length);
            Value = value;
        }

        public SyntaxToken(SyntaxKind kind, int position, int length, object value)
        {
            Kind  = kind;
            Span  = new TextSpan(position, length);
            Value = value;
        }

        public override SyntaxKind Kind { get; }
        public object Value { get; }
        public string[] CommentData { get; set; }

        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield break;
        }
    }

    public class ReturnSyntaxToken : SyntaxToken
    {
        public bool FollowedByLineBreak { get; }

        public ReturnSyntaxToken(int position, bool followedByLineBreak)
            : base(SyntaxKind.ReturnKeyword, position, "return", null)
        {
            FollowedByLineBreak = followedByLineBreak;
        }
    }
}