using System.Collections.Generic;
using System.Linq;
using Upsilon.Binder;
using Upsilon.Binder.VariableSymbols;

namespace Upsilon.Utilities
{
    public static class SearchHelper
    {
        public static BoundScope GetScopeAt(this BoundScript script, int characterPosition)
        {
            foreach (var node in script.GetNodeAtPosition(characterPosition))
            {
                if (node is IBoundNodeScopeOwner scopeOwner && scopeOwner.Scope != null)
                {
                    return scopeOwner.Scope;
                }
            }
            return script.Scope;
        }

        public static BoundNode GetBottomNodeAtPosition(this BoundNode node, int characterPosition)
        {
            return node.GetNodeAtPosition(characterPosition).First();
        }
        
        public static BoundStatement GetBottomStatementAtPosition(this BoundNode node, int characterPosition)
        {
            return (BoundStatement) node.GetNodeAtPosition(characterPosition).First(x => x is BoundStatement);
        }

        public static Dictionary<string, VariableSymbol> GetBoundScopeVisibleVariables(this BoundScope scope)
        {
            IEnumerable<KeyValuePair<string, VariableSymbol>> dictionary = scope.Variables;
            if (scope.ParentScope != null)
            {
                dictionary= dictionary.Union(scope.ParentScope.GetBoundScopeVisibleVariables());
            }

            if (scope.ReadOnlyScope != null)
            {
                dictionary = dictionary.Union(scope.ReadOnlyScope.GetBoundScopeVisibleVariables());
            }

            return dictionary.ToDictionary(k => k.Key, v => v.Value);
        }
    }
}